#!/bin/bash
#
# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e11|e12|e13|e14> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=boost
origpkgver=v1_63_0
pkgver=${origpkgver}
ssibuildshims_version=v0_15_05
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarver=`echo ${origpkgver} | sed -e 's/^v//'`
pkgtarname=boost_${pkgtarver}

get_this_dir

get_ssibuildshims

source define_basics

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    extra_command="variant=release"
elif [ "${extraqual}" = "debug" ]
then
    extra_command="variant=debug"
elif [ "${extraqual}" = "prof" ]
then
    extra_command="variant=profile"
fi

use_apple_clang=false
case ${basequal} in
    e7)
        cxxflg="-gdwarf-2 -std=c++1y -Wno-deprecated-declarations"
        ;;
    e9|e1[0124])
        cxxflg="-std=c++14 -Wno-deprecated-declarations"
        ;;
    e13)   # apple clang
        cxxflg="-std=c++14 -Wno-deprecated-declarations"
	use_apple_clang=true
        ;;
#    i1)
#        cxxflg="-gdwarf-2 -std=c++11"
#        ;;
    *)
        ssi_die "Unrecognized basequal ${basequal}"
esac

if [ ${OS1} = "Darwin" ]
then
  cxxflg="${cxxflg} -fvisibility-inlines-hidden"
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

set -x


# this is for building
cd "${blddir}"
tar xf "${tardir}/${pkgtarname}.tar.bz2"
cd "${pkgtarname}"

# this will be distributed for debugging
cd "${sourcedir}"
# Make sure the source code directory tree is readable
tar xf "${tardir}/${pkgtarname}.tar.bz2" && \
  find "${pkgtarname}" -type d -exec chmod 0755 \{\} \;

cd "${pkgtarname}"

cd "${blddir}/${pkgtarname}"
./bootstrap.sh --prefix="${pkgdir}" --without-icu --without-libraries=python,mpi

if [ ${OS1} = "Darwin" ]
then
  if [[ ${use_apple_clang} == "false" ]]; then
    mv project-config.jam project-config.jam.orig
    sed s%GCCDIR%"${GCC_FQ_DIR}"%g "${patchdir}/Darwin-project-config.jam" | sed s%BOOSTDIR%"${pkgdir}"%g > "${blddir}/${pkgtarname}/project-config.jam"
  fi
fi

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

# --debug-building  provides verbose output
# -d+2 echoes build commands
if [ -z "${cxxflg}" ]
then
  ./b2 -j ${num_cores} ${useicc:+"toolset=intel"} "${extra_command}"
else
  ./b2 -j ${num_cores} ${useicc:+"toolset=intel"} cxxflags="${cxxflg}" "${extra_command}"
fi
(( $? == 0 )) || ssi_die "ERROR: Build failed."

./b2 -j ${num_cores} ${useicc:+"toolset=intel"} "${extra_command}" install || ssi_die "ERROR: install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${BOOST_FQ_DIR}"
echo "BOOST_LIB is ${BOOST_LIB}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0

